Package: rsi
Title: Efficiently Retrieve and Process Satellite Imagery
Version: 0.3.3
Authors@R: c(
    person("Michael", "Mahoney", , "mike.mahoney.218@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2402-304X")),
        person("Felipe", "Carvalho", role = "rev",
           comment = "Felipe reviewed the package (v. 0.3.0) for rOpenSci, see <https://github.com/ropensci/software-review/issues/636>"),
    person("Michael", "Sumner", role = "rev",
           comment = "Michael reviewed the package (v. 0.3.0) for rOpenSci, see <https://github.com/ropensci/software-review/issues/636>"),
    person("Permian Global", role = c("cph", "fnd"))
  )
Description: Downloads spatial data from spatiotemporal asset catalogs 
    ('STAC'), computes standard spectral indices from the Awesome Spectral 
    Indices project (Montero et al. (2023) <doi:10.1038/s41597-023-02096-0>) 
    against raster data, and glues the outputs together into predictor bricks. 
    Methods focus on interoperability with the broader spatial ecosystem; 
    function arguments and outputs use classes from 'sf' and 'terra', and data 
    downloading functions support complex 'CQL2' queries using 'rstac'.
License: Apache License (>= 2)
Depends: R (>= 4.1)
Imports: future.apply, glue, httr, jsonlite, lifecycle,
        proceduralnames, rlang, rstac, sf, terra, tibble
Suggests: curl, knitr, progressr, rmarkdown, testthat (>= 3.0.0), withr
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
URL: https://github.com/Permian-Global-Research/rsi,
        https://permian-global-research.github.io/rsi/
BugReports: https://github.com/Permian-Global-Research/rsi/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-09 00:04:09 UTC; mikemahoney218
Author: Michael Mahoney [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2402-304X>),
  Felipe Carvalho [rev] (Felipe reviewed the package (v. 0.3.0) for
    rOpenSci, see
    <https://github.com/ropensci/software-review/issues/636>),
  Michael Sumner [rev] (Michael reviewed the package (v. 0.3.0) for
    rOpenSci, see
    <https://github.com/ropensci/software-review/issues/636>),
  Permian Global [cph, fnd]
Maintainer: Michael Mahoney <mike.mahoney.218@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-09 09:50:02 UTC
Built: R 4.4.3; ; 2026-04-09 13:38:16 UTC; unix
