# Generated by roxygen2: do not edit by hand

S3method("[",rsi_band_mapping)
S3method("[[",rsi_band_mapping)
S3method(c,rsi_band_mapping)
S3method(print,rsi_band_mapping)
export(alos_palsar_mask_function)
export(calculate_indices)
export(default_query_function)
export(filter_bands)
export(filter_platforms)
export(get_alos_palsar_imagery)
export(get_dem)
export(get_landsat_imagery)
export(get_naip_imagery)
export(get_sentinel1_imagery)
export(get_sentinel2_imagery)
export(get_stac_data)
export(landsat_mask_function)
export(landsat_platform_filter)
export(rsi_download_rasters)
export(rsi_gdal_config_options)
export(rsi_gdalwarp_options)
export(rsi_query_api)
export(sentinel2_mask_function)
export(sign_planetary_computer)
export(spectral_indices)
export(spectral_indices_url)
export(stack_rasters)
importFrom(httr,user_agent)
importFrom(rlang,"%||%")
importFrom(stats,predict)
